@@ -32,12 +32,22 @@
 
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 
-#include "debug.h"
-#include "sipe.h"
+#include <glib.h>
+
+#include "account.h"
+#include "eventloop.h"
+#include "network.h"
+#include "sslconn.h"
+
+#include "sipe-common.h"
+#include "sipmsg.h"
+#include "sip-sec.h"
+#include "sipe-backend.h"
 #include "sipe-utils.h"
-
 #include "http-conn.h"
+#include "sipe.h"
 
 /**
  * HTTP POST headers
@@ -49,7 +59,7 @@
 #define HTTP_CONN_POST_HEADER \
 "POST %s HTTP/1.1\r\n"\
 "Host: %s\r\n"\
-"User-Agent: Sipe/" SIPE_VERSION "\r\n"\
+"User-Agent: Sipe/" PACKAGE_VERSION "\r\n"\
 "Content-Length: %d\r\n"\
 "Content-Type: %s\r\n"
 
@@ -147,7 +157,7 @@ http_conn_invalidate_ssl_connection(HttpConn *http_conn);
 static void
 http_conn_close(HttpConn *http_conn, const char *message)
 {
-	purple_debug_info("sipe-http", "http_conn_close: closing http connection: %s\n", message ? message : "");
+	SIPE_DEBUG_INFO("http_conn_close: closing http connection: %s", message ? message : "");
 
 	http_conn_invalidate_ssl_connection(http_conn);
 	http_conn_free(http_conn);
@@ -165,24 +175,48 @@ http_conn_parse_url(const char *url,
 		    int *port,
 		    char **rel_url)
 {
-	char **parts = g_strsplit(url, "://", 2);
-	char *no_proto = parts[1] ? g_strdup(parts[1]) : g_strdup(parts[0]);
-	int port_tmp = sipe_strequal(parts[0], "https") ? 443 : 80;
-	char *tmp;
-	char *host_port;
+        char **parts = g_strsplit(url, "://", 2);
+        char *no_proto;
+        int port_tmp;
+        char *tmp;
+        char *host_port;
 
-	g_strfreev(parts);
-	tmp = strstr(no_proto, "/");
-	if (tmp && rel_url) *rel_url = g_strdup(tmp);
-	host_port = tmp ? g_strndup(no_proto, tmp - no_proto) : g_strdup(no_proto);
-	g_free(no_proto);
+        if(!parts) {
+                return;
+        } else if(!parts[0]) {
+                g_strfreev(parts);
+                return;
+        }
 
-	parts = g_strsplit(host_port, ":", 2);
-	if (host) *host = g_strdup(parts[0]);
-	if (port) *port = parts[1] ? atoi(parts[1]) : port_tmp;
-	g_strfreev(parts);
+        no_proto = parts[1] ? g_strdup(parts[1]) : g_strdup(parts[0]);
+        port_tmp = sipe_strequal(parts[0], "https") ? 443 : 80;
+        g_strfreev(parts);
 
-	g_free(host_port);
+        if(!no_proto) {
+		return;
+        }
+
+        tmp = strstr(no_proto, "/");
+        if (tmp && rel_url) *rel_url = g_strdup(tmp);
+        host_port = tmp ? g_strndup(no_proto, tmp - no_proto) : g_strdup(no_proto);
+        g_free(no_proto);
+
+        if(!host_port) {
+                return;
+        }
+
+        parts = g_strsplit(host_port, ":", 2);
+
+        if(parts) {
+                if (host) *host = g_strdup(parts[0]);
+                if(parts[0]) {
+			port_tmp = parts[1] ? atoi(parts[1]) : port_tmp;
+                }
+                if (port) *port = port_tmp;
+                g_strfreev(parts);
+        }
+
+        g_free(host_port);
 }
 
 static void
@@ -256,7 +290,7 @@ http_conn_input_cb_ssl(gpointer data,
 	gboolean firstread = TRUE;
 
 	if (conn == NULL) {
-		purple_debug_error("sipe-http", "Connection not found; Please try to connect again.\n");
+		SIPE_DEBUG_ERROR_NOFORMAT("Connection not found; Please try to connect again.");
 		return;
 	}
 
@@ -266,7 +300,7 @@ http_conn_input_cb_ssl(gpointer data,
 		if (conn->inbuflen < conn->inbufused + SIMPLE_BUF_INC) {
 			conn->inbuflen += SIMPLE_BUF_INC;
 			conn->inbuf = g_realloc(conn->inbuf, conn->inbuflen);
-			purple_debug_info("sipe-http", "http_conn_input_cb_ssl: new input buffer length %d\n", conn->inbuflen);
+			SIPE_DEBUG_INFO("http_conn_input_cb_ssl: new input buffer length %d", conn->inbuflen);
 		}
 
 		/* Try to read as much as there is space left in the buffer */
@@ -336,13 +370,13 @@ http_conn_create(PurpleAccount *account,
 	HttpConn *http_conn;
 
 	if (!full_url || (strlen(full_url) == 0)) {
-		purple_debug_info("sipe-http", "no URL supplied!\n");
+		SIPE_DEBUG_INFO_NOFORMAT("no URL supplied!");
 		return NULL;
 	}
 	if (sipe_strequal(conn_type, HTTP_CONN_SSL) &&
 	    !purple_ssl_is_supported())
 	{
-		purple_debug_info("sipe-http", "SSL support is not installed. Either install SSL support or configure a different connection type in the account editor\n");
+		SIPE_DEBUG_INFO_NOFORMAT("SSL support is not installed. Either install SSL support or configure a different connection type in the account editor.");
 		return NULL;
 	}
 
@@ -397,7 +431,7 @@ http_conn_process_input(HttpConn *http_conn)
 		time_t currtime = time(NULL);
 		cur += 2;
 		cur[0] = '\0';
-		purple_debug_info("sipe-http", "received - %s******\n%s\n******\n", ctime(&currtime), tmp = fix_newlines(conn->inbuf));
+		SIPE_DEBUG_INFO("received - %s******\n%s\n******", ctime(&currtime), tmp = fix_newlines(conn->inbuf));
 		g_free(tmp);
 
 		msg = sipmsg_parse_header(conn->inbuf);
@@ -414,14 +448,14 @@ http_conn_process_input(HttpConn *http_conn)
 			conn->inbufused = strlen(conn->inbuf);
 		} else {
 			if (msg){
-                           purple_debug_info("sipe-http", "process_input: body too short (%d < %d, strlen %d) - ignoring message\n", restlen, msg->bodylen, (int)strlen(conn->inbuf));
+                           SIPE_DEBUG_INFO("process_input: body too short (%d < %d, strlen %d) - ignoring message", restlen, msg->bodylen, (int)strlen(conn->inbuf));
 			sipmsg_free(msg);
                         }
 			return;
 		}
 
 		if (msg->body) {
-			purple_debug_info("sipe-http", "body:\n%s\n", msg->body);
+			SIPE_DEBUG_INFO("body:\n%s", msg->body);
 		}
 
 		http_conn_process_input_message(http_conn, msg);
@@ -443,11 +477,11 @@ http_conn_sendout_pkt(HttpConn *http_conn,
 	char *tmp;
 	int ret = 0;
 
-	purple_debug(PURPLE_DEBUG_MISC, "sipe-http", "sending - %s******\n%s\n******\n", ctime(&currtime), tmp = fix_newlines(buf));
+	SIPE_DEBUG_INFO("sending - %s******\n%s\n******", ctime(&currtime), tmp = fix_newlines(buf));
 	g_free(tmp);
 
 	if (http_conn->fd < 0) {
-		purple_debug_info("sipe-http", "http_conn_sendout_pkt: http_conn->fd < 0, exiting\n");
+		SIPE_DEBUG_INFO_NOFORMAT("http_conn_sendout_pkt: http_conn->fd < 0, exiting");
 		return;
 	}
 
@@ -458,12 +492,12 @@ http_conn_sendout_pkt(HttpConn *http_conn,
 	if (ret < 0 && errno == EAGAIN)
 		ret = 0;
 	else if (ret <= 0) { /* XXX: When does this happen legitimately? */
-		purple_debug_info("sipe-http", "http_conn_sendout_pkt: ret <= 0, exiting\n");
+		SIPE_DEBUG_INFO_NOFORMAT("http_conn_sendout_pkt: ret <= 0, exiting");
 		return;
 	}
 
 	if (ret < writelen) {
-		purple_debug_info("sipe-http", "http_conn_sendout_pkt: ret < writelen, exiting\n");
+		SIPE_DEBUG_INFO_NOFORMAT("http_conn_sendout_pkt: ret < writelen, exiting");
 	}
 }
 
@@ -496,7 +530,7 @@ http_conn_post(	HttpConn *http_conn,
 		void *data)
 {
 	if (!http_conn) {
-		purple_debug_info("sipe-http", "http_conn_post: NULL http_conn, exiting.\n");
+		SIPE_DEBUG_INFO_NOFORMAT("http_conn_post: NULL http_conn, exiting.");
 		return;
 	}
 
@@ -524,7 +558,7 @@ http_conn_process_input_message(HttpConn *http_conn,
 	{
 		const char *location = sipmsg_find_header(msg, "Location");
 
-		purple_debug_info("sipe-http", "http_conn_process_input_message: Redirect to: %s\n", location ? location : "");
+		SIPE_DEBUG_INFO("http_conn_process_input_message: Redirect to: %s", location ? location : "");
 
 		http_conn->do_close = http_conn_clone(http_conn);
 		http_conn->sec_ctx = NULL;
@@ -545,7 +579,7 @@ http_conn_process_input_message(HttpConn *http_conn,
 	else if (msg->response == 401) {
 		char *ptmp;
 #ifdef _WIN32
-#ifdef USE_KERBEROS
+#ifdef HAVE_LIBKRB5
 		char *tmp;
 #endif
 #endif
@@ -561,7 +595,7 @@ http_conn_process_input_message(HttpConn *http_conn,
 			if (http_conn->callback) {
 				(*http_conn->callback)(HTTP_CONN_ERROR_FATAL, NULL, http_conn, http_conn->data);
 			}
-			purple_debug_info("sipe-http", "http_conn_process_input_message: Authentication failed\n");
+			SIPE_DEBUG_INFO_NOFORMAT("http_conn_process_input_message: Authentication failed");
 			http_conn_set_close(http_conn);
 			return;
 		}
@@ -570,7 +604,7 @@ http_conn_process_input_message(HttpConn *http_conn,
 		auth_type = AUTH_TYPE_NTLM;
 		auth_name = "NTLM";
 #ifdef _WIN32
-#ifdef USE_KERBEROS
+#ifdef HAVE_LIBKRB5
 		tmp = sipmsg_find_auth_header(msg, "Negotiate");
 		if (tmp && http_conn->auth && http_conn->auth->use_negotiate) {
 			ptmp = tmp;
@@ -580,16 +614,16 @@ http_conn_process_input_message(HttpConn *http_conn,
 #endif
 #endif
 		if (!ptmp) {
-			purple_debug_info("sipe-http", "http_conn_process_input_message: Only %s supported in the moment, exiting\n",
+			SIPE_DEBUG_INFO("http_conn_process_input_message: Only %s supported in the moment, exiting",
 #ifdef _WIN32
-#ifdef USE_KERBEROS
+#ifdef HAVE_LIBKRB5
 				"NTLM and Negotiate authentications are"
-#else //USE_KERBEROS
+#else /* !HAVE_LIBKRB5 */
 				"NTLM authentication is"
-#endif //USE_KERBEROS
-#else //_WIN32
+#endif /* HAVE_LIBKRB5 */
+#else /* !_WIN32 */
 				"NTLM authentication is"
-#endif //_WIN32
+#endif /* _WIN32 */
 
 			);
 		}
@@ -620,7 +654,7 @@ http_conn_process_input_message(HttpConn *http_conn,
 			if (http_conn->callback) {
 				(*http_conn->callback)(HTTP_CONN_ERROR_FATAL, NULL, http_conn, http_conn->data);
 			}
-			purple_debug_info("sipe-http", "http_conn_process_input_message: Failed to initialize security context\n");
+			SIPE_DEBUG_INFO_NOFORMAT("http_conn_process_input_message: Failed to initialize security context");
 			http_conn_set_close(http_conn);
 			return;
 		}
