@@ -20,6 +20,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* Forward declarations */
+struct _PurpleAccount;
+
 #define HTTP_CONN_SSL  "SSL"
 #define HTTP_CONN_TCP  "TCP"
 
@@ -45,7 +48,7 @@ typedef void (*HttpConnCallback) (int return_code, const char *body,
  * Creates SSL connection and POST.
  */
 HttpConn *
-http_conn_create(PurpleAccount *account,
+http_conn_create(struct _PurpleAccount *account,
 		 const char *conn_type,
 		 const char *full_url,
 		 const char *body,
@@ -73,4 +76,3 @@ http_conn_set_close(HttpConn* http_conn);
 
 void
 http_conn_auth_free(struct http_conn_auth* auth);
-
