@@ -45,29 +45,22 @@
  *    not in NTLM (LE) structures.
  */
 
-#include <glib.h>
-#include <glib/gprintf.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
 #include <stdlib.h>
-#include "debug.h"
+#include <string.h>
+#include <time.h>
 
-#ifdef _WIN32
-#include "internal.h"
-#endif /* _WIN32 */
+#include <glib.h>
 
 #ifdef HAVE_LANGINFO_CODESET
 #include <langinfo.h>
 #endif /* HAVE_LANGINFO_CODESET */
 
-#include "util.h"
-#include "cipher.h"
-
-#include "sipe.h"
-#include "sipe-utils.h"
+#include "sipe-common.h"
+#include "sip-sec.h"
 #include "sip-sec-mech.h"
 #include "sip-sec-ntlm.h"
+#include "sipe-backend.h"
+#include "sipe-utils.h"
 
 /* [MS-NLMP] */
 #define NTLMSSP_NEGOTIATE_UNICODE			0x00000001	/* A  */
@@ -188,8 +181,6 @@ struct version test_version;		/* hard-coded in implementation */
 #define NTLMSSP_LN_OR_NT_KEY_LEN  16
 #define NTLMSSP_LM_RESP_LEN 24
 #define NTLMSSP_SESSION_KEY_LEN  16
-#define MD4_DIGEST_LEN 16
-#define MD5_DIGEST_LEN 16
 
 #define IS_FLAG(flags, flag) (((flags) & (flag)) == (flag))
 
@@ -384,26 +375,13 @@ static void setup_des_key(const unsigned char key_56[], unsigned char *key)
 	key[7] =  (key_56[6] << 1) & 0xFF;
 }
 
-static void des_ecb_encrypt(const unsigned char *plaintext, unsigned char *result, const unsigned char *key)
-{
-	PurpleCipher *cipher;
-	PurpleCipherContext *context;
-	size_t outlen;
-
-	cipher = purple_ciphers_find_cipher("des");
-	context = purple_cipher_context_new(cipher, NULL);
-	purple_cipher_context_set_key(context, (guchar*)key);
-	purple_cipher_context_encrypt(context, (guchar*)plaintext, 8, (guchar*)result, &outlen);
-	purple_cipher_context_destroy(context);
-}
-
 /* (k = 7 byte key, d = 8 byte data) returns 8 bytes in results */
 static void
 DES (const unsigned char *k, const unsigned char *d, unsigned char * results)
 {
 	unsigned char key[8];
 	setup_des_key(k, key);
-	des_ecb_encrypt(d, results, key);
+	sipe_backend_encrypt_des(key, d, 8, results);
 }
 
 /* (K = 21 byte key, D = 8 bytes of data) returns 24 bytes in results: */
@@ -424,37 +402,14 @@ DESL (const unsigned char *k, const unsigned char *d, unsigned char * results)
 }
 #endif
 
-static void
-RC4K (const unsigned char * k, unsigned long key_len, const unsigned char * d, int len, unsigned char * result)
-{
-	PurpleCipherContext * context = purple_cipher_context_new_by_name("rc4", NULL);
-	purple_cipher_context_set_option(context, "key_len", (gpointer)key_len);
-	purple_cipher_context_set_key(context, k);
-	purple_cipher_context_encrypt(context, (const guchar *)d, len, result, NULL);
-	purple_cipher_context_destroy(context);
-}
+#define RC4K(key, key_len, plain, plain_len, encrypted) \
+	sipe_backend_encrypt_rc4((key), (key_len), (plain), (plain_len), (encrypted))
 
 /* out 16 bytes */
-static void
-MD4 (const unsigned char * d, int len, unsigned char * result)
-{
-	PurpleCipher * cipher = purple_ciphers_find_cipher("md4");
-	PurpleCipherContext * context = purple_cipher_context_new(cipher, NULL);
-	purple_cipher_context_append(context, (guchar*)d, len);
-	purple_cipher_context_digest(context, MD4_DIGEST_LEN, (guchar*)result, NULL);
-	purple_cipher_context_destroy(context);
-}
+#define MD4(d, len, result) sipe_backend_digest_md4((d), (len), (result))
 
 /* out 16 bytes */
-static void
-MD5 (const unsigned char * d, int len, unsigned char * result)
-{
-	PurpleCipher * cipher = purple_ciphers_find_cipher("md5");
-	PurpleCipherContext * context = purple_cipher_context_new(cipher, NULL);
-	purple_cipher_context_append(context, (guchar*)d, len);
-	purple_cipher_context_digest(context, MD5_DIGEST_LEN, (guchar*)result, NULL);
-	purple_cipher_context_destroy(context);
-}
+#define MD5(d, len, result) sipe_backend_digest_md5((d), (len), (result))
 
 /* out 16 bytes */
 /*
@@ -486,19 +441,8 @@ HMACT64 (const unsigned char *key, int key_len, const unsigned char *data, int d
 */
 
 /* out 16 bytes */
-static void
-HMAC_MD5 (const unsigned char *key, int key_len, const unsigned char *data, int data_len, unsigned char *result)
-{
-	PurpleCipher *cipher = purple_ciphers_find_cipher("hmac");
-	PurpleCipherContext *context = purple_cipher_context_new(cipher, NULL);
-
-	purple_cipher_context_set_option(context, "hash", "md5");
-	purple_cipher_context_set_key_with_len(context, (guchar *)key, (key_len));
-
-	purple_cipher_context_append(context, (guchar *)data, data_len);
-	purple_cipher_context_digest(context, 16, (guchar*)result, NULL);
-	purple_cipher_context_destroy(context);
-}
+#define HMAC_MD5(key, key_len, data, data_len, result) \
+	sipe_backend_digest_hmac_md5((key), (key_len), (data), (data_len), (result))
 
 /* NTLM Core Methods */
 
@@ -807,13 +751,13 @@ SEALKEY (guint32 flags, const unsigned char * random_session_key, gboolean clien
 		int key_len;
 
 		if (IS_FLAG(flags, NTLMSSP_NEGOTIATE_128)) {
-			purple_debug_info("sipe", "NTLM SEALKEY(): 128-bit key (Extended session security)\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 128-bit key (Extended session security)");
 			key_len = 16;
 		} else if (IS_FLAG(flags, NTLMSSP_NEGOTIATE_56)) {
-			purple_debug_info("sipe", "NTLM SEALKEY(): 56-bit key (Extended session security)\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 56-bit key (Extended session security)");
 			key_len = 7;
 		} else {
-			purple_debug_info("sipe", "NTLM SEALKEY(): 40-bit key (Extended session security)\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 40-bit key (Extended session security)");
 			key_len = 5;
 		}
 
@@ -825,11 +769,11 @@ SEALKEY (guint32 flags, const unsigned char * random_session_key, gboolean clien
 	else if (IS_FLAG(flags, NTLMSSP_NEGOTIATE_LM_KEY)) /* http://davenport.sourceforge.net/ntlm.html#ntlm1KeyWeakening */
 	{
 		if (IS_FLAG(flags, NTLMSSP_NEGOTIATE_56)) {
-			purple_debug_info("sipe", "NTLM SEALKEY(): 56-bit key\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 56-bit key");
 			memcpy(result, random_session_key, 7);
 			result[7] = 0xA0;
 		} else {
-			purple_debug_info("sipe", "NTLM SEALKEY(): 40-bit key\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 40-bit key");
 			memcpy(result, random_session_key, 5);
 			result[5] = 0xE5;
 			result[6] = 0x38;
@@ -838,7 +782,7 @@ SEALKEY (guint32 flags, const unsigned char * random_session_key, gboolean clien
 	}
 	else
 	{
-		purple_debug_info("sipe", "NTLM SEALKEY(): 128-bit key\n");
+		SIPE_DEBUG_INFO_NOFORMAT("NTLM SEALKEY(): 128-bit key");
 		memcpy(result, random_session_key, 16);
 	}
 }
@@ -913,7 +857,7 @@ MAC (guint32 flags,
 			memcpy(seal_key_, seal_key, seal_key_len);
 		}
 
-		purple_debug_info("sipe", "NTLM MAC(): Extented Session Security\n");
+		SIPE_DEBUG_INFO_NOFORMAT("NTLM MAC(): Extented Session Security");
 
 		res_ptr = (guint32 *)result;
 		res_ptr[0] = GUINT32_TO_LE(1); // 4 bytes
@@ -926,10 +870,10 @@ MAC (guint32 flags,
 		HMAC_MD5(sign_key, sign_key_len, tmp, 4 + buf_len, hmac);
 
 		if (IS_FLAG(flags, NTLMSSP_NEGOTIATE_KEY_EXCH)) {
-			purple_debug_info("sipe", "NTLM MAC(): Key Exchange\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM MAC(): Key Exchange");
 			RC4K(seal_key_, seal_key_len, hmac, 8, result+4);
 		} else {
-			purple_debug_info("sipe", "NTLM MAC(): *NO* Key Exchange\n");
+			SIPE_DEBUG_INFO_NOFORMAT("NTLM MAC(): *NO* Key Exchange");
 			memcpy(result+4, hmac, 8);
 		}
 	} else {
@@ -941,7 +885,7 @@ MAC (guint32 flags,
 			GUINT32_TO_LE(sequence)
 		}; // 4, 4, 4 bytes
 
-		purple_debug_info("sipe", "NTLM MAC(): *NO* Extented Session Security\n");
+		SIPE_DEBUG_INFO_NOFORMAT("NTLM MAC(): *NO* Extented Session Security");
 
 		RC4K(seal_key, seal_key_len, (const guchar *)plaintext, 12, result+4);
 
@@ -1070,7 +1014,7 @@ sip_sec_ntlm_gen_authenticate(guchar **client_sign_key,
 	if (!IS_FLAG(*flags, NEGOTIATE_FLAGS_COMMON_MIN) ||
 	    !(is_connection_based || IS_FLAG(*flags, NEGOTIATE_FLAGS_CONNLESS_EXTRA)))
 	{
-		purple_debug_info("sipe", "sip_sec_ntlm_gen_authenticate: received incompatible NTLM NegotiateFlags, exiting.");
+		SIPE_DEBUG_INFO_NOFORMAT("sip_sec_ntlm_gen_authenticate: received incompatible NTLM NegotiateFlags, exiting.");
 		return SIP_SEC_E_INTERNAL_ERROR;
 	}
 
@@ -1127,7 +1071,7 @@ sip_sec_ntlm_gen_authenticate(guchar **client_sign_key,
 	}
 
 	tmp = buff_to_hex_str(exported_session_key, 16);
-	purple_debug_info("sipe", "NTLM AUTHENTICATE: exported session key (not encrypted): %s\n", tmp);
+	SIPE_DEBUG_INFO("NTLM AUTHENTICATE: exported session key (not encrypted): %s", tmp);
 	g_free(tmp);
 
 	if (IS_FLAG(neg_flags, NTLMSSP_NEGOTIATE_SIGN) ||
@@ -1319,7 +1263,7 @@ sip_sec_ntlm_sipe_signature_make(guint32 flags,
 	MAC(flags,  msg,strlen(msg),  sign_key,16,  seal_key,16,  random_pad, 100, result);
 
 	res = buff_to_hex_str(result, 16);
-	purple_debug_info("sipe", "NTLM calculated MAC: %s\n", res);
+	SIPE_DEBUG_INFO("NTLM calculated MAC: %s", res);
 	g_free(res);
 }
 
@@ -1746,7 +1690,7 @@ sip_sec_init_sec_context__ntlm(SipSecContext context,
 {
 	context_ntlm ctx = (context_ntlm) context;
 
-	purple_debug_info("sipe", "sip_sec_init_sec_context__ntlm: in use\n");
+	SIPE_DEBUG_INFO_NOFORMAT("sip_sec_init_sec_context__ntlm: in use");
 
 	ctx->step++;
 	if (ctx->step == 1) {
@@ -1789,7 +1733,7 @@ sip_sec_init_sec_context__ntlm(SipSecContext context,
 					      &server_seal_key,
 					      ctx->username,
 					      ctx->password,
-					      (tmp = g_ascii_strup(sipe_get_host_name(), -1)),
+					      (tmp = g_ascii_strup(g_get_host_name(), -1)),
 					      ctx->domain,
 					      server_challenge,
 					      time_val,
@@ -1915,14 +1859,14 @@ void sip_sec_init__ntlm(void)
 
 	convert_from_utf16le = g_iconv_open(sys_cp, "UTF-16LE");
 	if (convert_from_utf16le == (GIConv)-1) {
-		purple_debug_error("sipe", "g_iconv_open from UTF-16LE to %s failed\n",
-				   sys_cp);
+		SIPE_DEBUG_ERROR("g_iconv_open from UTF-16LE to %s failed",
+				 sys_cp);
 	}
 
 	convert_to_utf16le = g_iconv_open("UTF-16LE", sys_cp);
 	if (convert_from_utf16le == (GIConv)-1) {
-		purple_debug_error("sipe", "g_iconv_open from %s to UTF-16LE failed\n",
-				   sys_cp);
+		SIPE_DEBUG_ERROR("g_iconv_open from %s to UTF-16LE failed",
+				 sys_cp);
 	}
 }
 
