@@ -3,6 +3,7 @@
  *
  * pidgin-sipe
  *
+ * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2009 pier11 <pier11@operamail.com>
  *
  *
@@ -21,12 +22,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define SIPE_UNUSED_PARAMETER __attribute__((unused))
-
 /* Opaque type definition for security context */
 typedef struct sip_sec_context *SipSecContext;
 
-typedef enum
+typedef enum sip_sec_auth_type
 {
 	AUTH_TYPE_UNSET = 0,
 	AUTH_TYPE_DIGEST,
@@ -35,7 +34,7 @@ typedef enum
 	AUTH_TYPE_NEGOTIATE
 } SipSecAuthType;
 
-//// Sipe convenience methods ////
+/*** Sipe convenience methods ***/
 
 /**
  * Initializes Sipe security context.
@@ -140,7 +139,7 @@ int sip_sec_verify_signature(SipSecContext context,
 
 /**
  * Initialize & destroy functions for sip-sec.
- * Should be called on loading and unloading of the plugin.
+ * Should be called on loading and unloading of the core.
  */
 void sip_sec_init(void);
 void sip_sec_destroy(void);
