@@ -28,15 +28,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+
 #include <glib.h>
 
-#include "debug.h"
-
-#include "sipe.h"
+#include "sip-sec.h"
+#include "sipe-backend.h"
 #include "sipe-cal.h"
-#include "sipe-utils.h"
 #include "sipe-nls.h"
-
+#include "sipe-utils.h"
+#include "sipe-xml.h"
+#include "sipe.h"
 
 #define TIME_NULL   (time_t)-1
 #define IS(time)    (time != TIME_NULL)
@@ -183,31 +184,29 @@ sipe_cal_event_hash(struct sipe_cal_event* event)
 				event->is_meeting);
 }
 
-static void
-sipe_setenv(const char *name,
-	    const char *value)
+#define ENVIRONMENT_TIMEZONE "TZ"
+
+static gchar *
+sipe_switch_tz(const char *tz)
 {
-#ifdef HAVE_SETENV
-	setenv(name, value, 1);
-#else
-	int len = strlen(name) + 1 + strlen(value) + 1;
-	char *str = g_malloc0(len);
-	sprintf(str, "%s=%s", name, value);
-	putenv(str);
-#endif
+	gchar *tz_orig;
+
+	tz_orig = g_strdup(g_getenv(ENVIRONMENT_TIMEZONE));
+	g_setenv(ENVIRONMENT_TIMEZONE, tz, TRUE);
+	tzset();
+	return(tz_orig);
 }
 
 static void
-sipe_unsetenv(const char *name)
+sipe_reset_tz(gchar *tz_orig)
 {
-#ifdef HAVE_UNSETENV
-	unsetenv(name);
-#else
-	int len = strlen(name) + 1 + 1;
-	char *str = g_malloc0(len);
-	sprintf(str, "%s=", name);
-	putenv(str);
-#endif
+	if (tz_orig) {
+		g_setenv(ENVIRONMENT_TIMEZONE, tz_orig, TRUE);
+		g_free(tz_orig);
+	} else {
+		g_unsetenv(ENVIRONMENT_TIMEZONE);
+	}
+	tzset();
 }
 
 /**
@@ -222,20 +221,11 @@ sipe_mktime_tz(struct tm *tm,
 	       const char* tz)
 {
 	time_t ret;
-	char *tz_old;
-
-	tz_old = getenv("TZ");
-	sipe_setenv("TZ", tz);
-	tzset();
+	gchar *tz_orig;
 
+	tz_orig = sipe_switch_tz(tz);
 	ret = mktime(tm);
-
-	if (tz_old) {
-		sipe_setenv("TZ", tz_old);
-	} else {
-		sipe_unsetenv("TZ");
-	}
-	tzset();
+	sipe_reset_tz(tz_orig);
 
 	return ret;
 }
@@ -252,20 +242,11 @@ sipe_localtime_tz(const time_t *time,
 		  const char* tz)
 {
 	struct tm *ret;
-	char *tz_old;
-
-	tz_old = getenv("TZ");
-	sipe_setenv("TZ", tz);
-	tzset();
+	gchar *tz_orig;
 
+	tz_orig = sipe_switch_tz(tz);
 	ret = localtime(time);
-
-	if (tz_old) {
-		sipe_setenv("TZ", tz_old);
-	} else {
-		sipe_unsetenv("TZ");
-	}
-	tzset();
+	sipe_reset_tz(tz_orig);
 
 	return ret;
 }
@@ -346,10 +327,10 @@ sipe_cal_get_std_dst_time(time_t now,
 }
 
 static void
-sipe_cal_parse_std_dst(xmlnode *xn_std_dst_time,
-		       struct sipe_cal_std_dst* std_dst)
+sipe_cal_parse_std_dst(const sipe_xml *xn_std_dst_time,
+		       struct sipe_cal_std_dst *std_dst)
 {
-	xmlnode *node;
+	const sipe_xml *node;
 	gchar *tmp;
 
 	if (!xn_std_dst_time) return;
@@ -364,42 +345,43 @@ sipe_cal_parse_std_dst(xmlnode *xn_std_dst_time,
     </StandardTime>
 */
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "Bias"))) {
-		std_dst->bias = atoi(tmp = xmlnode_get_data(node));
+	if ((node = sipe_xml_child(xn_std_dst_time, "Bias"))) {
+		std_dst->bias = atoi(tmp = sipe_xml_data(node));
 		g_free(tmp);
 	}
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "Time"))) {
-		std_dst->time = xmlnode_get_data(node);
+	if ((node = sipe_xml_child(xn_std_dst_time, "Time"))) {
+		std_dst->time = sipe_xml_data(node);
 	}
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "DayOrder"))) {
-		std_dst->day_order = atoi(tmp = xmlnode_get_data(node));
+	if ((node = sipe_xml_child(xn_std_dst_time, "DayOrder"))) {
+		std_dst->day_order = atoi(tmp = sipe_xml_data(node));
 		g_free(tmp);
 	}
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "Month"))) {
-		std_dst->month = atoi(tmp = xmlnode_get_data(node));
+	if ((node = sipe_xml_child(xn_std_dst_time, "Month"))) {
+		std_dst->month = atoi(tmp = sipe_xml_data(node));
 		g_free(tmp);
 	}
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "DayOfWeek"))) {
-		std_dst->day_of_week = xmlnode_get_data(node);
+	if ((node = sipe_xml_child(xn_std_dst_time, "DayOfWeek"))) {
+		std_dst->day_of_week = sipe_xml_data(node);
 	}
 
-	if ((node = xmlnode_get_child(xn_std_dst_time, "Year"))) {
-		std_dst->year = xmlnode_get_data(node);
+	if ((node = sipe_xml_child(xn_std_dst_time, "Year"))) {
+		std_dst->year = sipe_xml_data(node);
 	}
 }
 
 void
-sipe_cal_parse_working_hours(xmlnode *xn_working_hours,
+sipe_cal_parse_working_hours(const sipe_xml *xn_working_hours,
 			     struct sipe_buddy *buddy)
 {
-	xmlnode *xn_bias;
-	xmlnode *xn_working_period;
-	xmlnode *xn_standard_time;
-	xmlnode *xn_daylight_time;
+	const sipe_xml *xn_bias;
+	const sipe_xml *xn_timezone;
+	const sipe_xml *xn_working_period;
+	const sipe_xml *xn_standard_time;
+	const sipe_xml *xn_daylight_time;
 	gchar *tmp;
 	time_t now = time(NULL);
 	struct sipe_cal_std_dst* std;
@@ -424,31 +406,32 @@ sipe_cal_parse_working_hours(xmlnode *xn_working_hours,
 	sipe_cal_free_working_hours(buddy->cal_working_hours);
 	buddy->cal_working_hours = g_new0(struct sipe_cal_working_hours, 1);
 
-	xn_bias = xmlnode_get_descendant(xn_working_hours, "TimeZone", "Bias", NULL);
+	xn_timezone = sipe_xml_child(xn_working_hours, "TimeZone");
+	xn_bias = sipe_xml_child(xn_timezone, "Bias");
 	if (xn_bias) {
-		buddy->cal_working_hours->bias = atoi(tmp = xmlnode_get_data(xn_bias));
+		buddy->cal_working_hours->bias = atoi(tmp = sipe_xml_data(xn_bias));
 		g_free(tmp);
 	}
 
-	xn_standard_time = xmlnode_get_descendant(xn_working_hours, "TimeZone", "StandardTime", NULL);
-	xn_daylight_time = xmlnode_get_descendant(xn_working_hours, "TimeZone", "DaylightTime", NULL);
+	xn_standard_time = sipe_xml_child(xn_timezone, "StandardTime");
+	xn_daylight_time = sipe_xml_child(xn_timezone, "DaylightTime");
 
 	std = &((*buddy->cal_working_hours).std);
 	dst = &((*buddy->cal_working_hours).dst);
 	sipe_cal_parse_std_dst(xn_standard_time, std);
 	sipe_cal_parse_std_dst(xn_daylight_time, dst);
 
-	xn_working_period = xmlnode_get_descendant(xn_working_hours, "WorkingPeriodArray", "WorkingPeriod", NULL);
+	xn_working_period = sipe_xml_child(xn_working_hours, "WorkingPeriodArray/WorkingPeriod");
 	if (xn_working_period) {
 		buddy->cal_working_hours->days_of_week =
-			xmlnode_get_data(xmlnode_get_child(xn_working_period, "DayOfWeek"));
+			sipe_xml_data(sipe_xml_child(xn_working_period, "DayOfWeek"));
 
 		buddy->cal_working_hours->start_time =
-			atoi(tmp = xmlnode_get_data(xmlnode_get_child(xn_working_period, "StartTimeInMinutes")));
+			atoi(tmp = sipe_xml_data(sipe_xml_child(xn_working_period, "StartTimeInMinutes")));
 		g_free(tmp);
 
 		buddy->cal_working_hours->end_time =
-			atoi(tmp = xmlnode_get_data(xmlnode_get_child(xn_working_period, "EndTimeInMinutes")));
+			atoi(tmp = sipe_xml_data(sipe_xml_child(xn_working_period, "EndTimeInMinutes")));
 		g_free(tmp);
 	}
 
@@ -579,16 +562,16 @@ sipe_cal_get_status(struct sipe_buddy *buddy,
 	int index;
 
 	if (!buddy || !buddy->cal_start_time || !buddy->cal_granularity) {
-		purple_debug_info("sipe", "sipe_cal_get_status: no calendar data1 for %s, exiting\n",
+		SIPE_DEBUG_INFO("sipe_cal_get_status: no calendar data1 for %s, exiting",
 				  buddy ? (buddy->name ? buddy->name : "") : "");
 		return SIPE_CAL_NO_DATA;
 	}
 
 	if (!(free_busy = sipe_cal_get_free_busy(buddy))) {
-		purple_debug_info("sipe", "sipe_cal_get_status: no calendar data2 for %s, exiting\n", buddy->name);
+		SIPE_DEBUG_INFO("sipe_cal_get_status: no calendar data2 for %s, exiting", buddy->name);
 		return SIPE_CAL_NO_DATA;
 	}
-	purple_debug_info("sipe", "sipe_cal_get_description: buddy->cal_free_busy=\n%s\n", free_busy);
+	SIPE_DEBUG_INFO("sipe_cal_get_description: buddy->cal_free_busy=\n%s", free_busy);
 
 	cal_start = sipe_utils_str_to_time(buddy->cal_start_time);
 
@@ -768,7 +751,7 @@ sipe_cal_get_free_busy(struct sipe_buddy *buddy)
 		gsize i;
 		int j = 0;
 
-		cal_dec64 = purple_base64_decode(buddy->cal_free_busy_base64, &cal_dec64_len);
+		cal_dec64 = g_base64_decode(buddy->cal_free_busy_base64, &cal_dec64_len);
 
 		buddy->cal_free_busy = g_malloc0(cal_dec64_len * 4 + 1);
 /*
@@ -824,7 +807,7 @@ sipe_cal_get_freebusy_base64(const char* freebusy_hex)
 		}
 	}
 
-	res_base64 = purple_base64_encode(res, shift_factor ? res_len : res_len - 1);
+	res_base64 = g_base64_encode(res, shift_factor ? res_len : res_len - 1);
 	g_free(res);
 	return res_base64;
 }
@@ -852,16 +835,16 @@ sipe_cal_get_description(struct sipe_buddy *buddy)
 				    _("No data")};
 
 	if (buddy->cal_granularity != 15) {
-		purple_debug_info("sipe", "sipe_cal_get_description: granularity %d is unsupported, exiting.\n", buddy->cal_granularity);
+		SIPE_DEBUG_INFO("sipe_cal_get_description: granularity %d is unsupported, exiting.", buddy->cal_granularity);
 		return NULL;
 	}
 
 	/* to lazy load if needed */
 	free_busy = sipe_cal_get_free_busy(buddy);
-	purple_debug_info("sipe", "sipe_cal_get_description: buddy->cal_free_busy=\n%s\n", free_busy ? free_busy : "");
+	SIPE_DEBUG_INFO("sipe_cal_get_description: buddy->cal_free_busy=\n%s", free_busy ? free_busy : "");
 
 	if (!buddy->cal_free_busy || !buddy->cal_granularity || !buddy->cal_start_time) {
-		purple_debug_info("sipe", "sipe_cal_get_description: no calendar data, exiting");
+		SIPE_DEBUG_INFO_NOFORMAT("sipe_cal_get_description: no calendar data, exiting");
 		return NULL;
 	}
 
@@ -870,40 +853,40 @@ sipe_cal_get_description(struct sipe_buddy *buddy)
 
 	current_cal_state = sipe_cal_get_status0(free_busy, cal_start, buddy->cal_granularity, time(NULL), &index);
 	if (current_cal_state == SIPE_CAL_NO_DATA) {
-		purple_debug_info("sipe", "sipe_cal_get_description: calendar is undefined for present moment, exiting.\n");
+		SIPE_DEBUG_INFO_NOFORMAT("sipe_cal_get_description: calendar is undefined for present moment, exiting.");
 		return NULL;
 	}
 
 	switch_time = sipe_cal_get_switch_time(free_busy, cal_start, buddy->cal_granularity, index, current_cal_state, &to_state);
 
-	purple_debug_info("sipe", "\n* Calendar *\n");
+	SIPE_DEBUG_INFO_NOFORMAT("\n* Calendar *");
 	if (buddy->cal_working_hours) {
 		sipe_cal_get_today_work_hours(buddy->cal_working_hours, &start, &end, &next_start);
 
-		purple_debug_info("sipe", "Remote now timezone : %s\n", sipe_cal_get_tz(buddy->cal_working_hours, now));
-		purple_debug_info("sipe", "std.switch_time(GMT): %s",
-				IS((*buddy->cal_working_hours).std.switch_time) ? asctime(gmtime(&((*buddy->cal_working_hours).std.switch_time))) : "\n");
-		purple_debug_info("sipe", "dst.switch_time(GMT): %s",
-				IS((*buddy->cal_working_hours).dst.switch_time) ? asctime(gmtime(&((*buddy->cal_working_hours).dst.switch_time))) : "\n");
-		purple_debug_info("sipe", "Remote now time     : %s",
+		SIPE_DEBUG_INFO("Remote now timezone : %s", sipe_cal_get_tz(buddy->cal_working_hours, now));
+		SIPE_DEBUG_INFO("std.switch_time(GMT): %s",
+				IS((*buddy->cal_working_hours).std.switch_time) ? asctime(gmtime(&((*buddy->cal_working_hours).std.switch_time))) : "");
+		SIPE_DEBUG_INFO("dst.switch_time(GMT): %s",
+				IS((*buddy->cal_working_hours).dst.switch_time) ? asctime(gmtime(&((*buddy->cal_working_hours).dst.switch_time))) : "");
+		SIPE_DEBUG_INFO("Remote now time     : %s",
 			asctime(sipe_localtime_tz(&now, sipe_cal_get_tz(buddy->cal_working_hours, now))));
-		purple_debug_info("sipe", "Remote start time   : %s",
-			IS(start) ? asctime(sipe_localtime_tz(&start, sipe_cal_get_tz(buddy->cal_working_hours, start))) : "\n");
-		purple_debug_info("sipe", "Remote end time     : %s",
-			IS(end) ? asctime(sipe_localtime_tz(&end, sipe_cal_get_tz(buddy->cal_working_hours, end))) : "\n");
-		purple_debug_info("sipe", "Rem. next_start time: %s",
-			IS(next_start) ? asctime(sipe_localtime_tz(&next_start, sipe_cal_get_tz(buddy->cal_working_hours, next_start))) : "\n");
-		purple_debug_info("sipe", "Remote switch time  : %s",
-			IS(switch_time) ? asctime(sipe_localtime_tz(&switch_time, sipe_cal_get_tz(buddy->cal_working_hours, switch_time))) : "\n");
+		SIPE_DEBUG_INFO("Remote start time   : %s",
+			IS(start) ? asctime(sipe_localtime_tz(&start, sipe_cal_get_tz(buddy->cal_working_hours, start))) : "");
+		SIPE_DEBUG_INFO("Remote end time     : %s",
+			IS(end) ? asctime(sipe_localtime_tz(&end, sipe_cal_get_tz(buddy->cal_working_hours, end))) : "");
+		SIPE_DEBUG_INFO("Rem. next_start time: %s",
+			IS(next_start) ? asctime(sipe_localtime_tz(&next_start, sipe_cal_get_tz(buddy->cal_working_hours, next_start))) : "");
+		SIPE_DEBUG_INFO("Remote switch time  : %s",
+			IS(switch_time) ? asctime(sipe_localtime_tz(&switch_time, sipe_cal_get_tz(buddy->cal_working_hours, switch_time))) : "");
 	} else {
-		purple_debug_info("sipe", "Local now time      : %s",
+		SIPE_DEBUG_INFO("Local now time      : %s",
 			asctime(localtime(&now)));
-		purple_debug_info("sipe", "Local switch time   : %s",
-			IS(switch_time) ? asctime(localtime(&switch_time)) : "\n");
+		SIPE_DEBUG_INFO("Local switch time   : %s",
+			IS(switch_time) ? asctime(localtime(&switch_time)) : "");
 	}
-	purple_debug_info("sipe", "Calendar End (GMT)  : %s", asctime(gmtime(&cal_end)));
-	purple_debug_info("sipe", "current cal state   : %s\n", cal_states[current_cal_state]);
-	purple_debug_info("sipe", "switch  cal state   : %s\n", cal_states[to_state]         );
+	SIPE_DEBUG_INFO("Calendar End (GMT)  : %s", asctime(gmtime(&cal_end)));
+	SIPE_DEBUG_INFO("current cal state   : %s", cal_states[current_cal_state]);
+	SIPE_DEBUG_INFO("switch  cal state   : %s", cal_states[to_state]         );
 
 	/* Calendar: string calculations */
 
