@@ -20,16 +20,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <stdlib.h>
 #include <string.h>
+#include <time.h>
+
 #include <glib.h>
 
-#include "debug.h"
-
-#include "sipe.h"
-#include "sipe-utils.h"
+#include "sipe-common.h"
+#include "sipmsg.h"
+#include "sipe-backend.h"
 #include "sipe-dialog.h"
 #include "sipe-session.h"
-#include "sipmsg.h"
+#include "sipe-utils.h"
 
 void sipe_dialog_free(struct sip_dialog *dialog)
 {
@@ -92,12 +94,12 @@ sipe_dialog_find_3(struct sip_session *session,
 				dialog->ourtag &&
 				dialog->theirtag &&
 
-				!g_ascii_strcasecmp(dialog_in->callid, dialog->callid) &&
-				!g_ascii_strcasecmp(dialog_in->ourtag, dialog->ourtag) &&
-				!g_ascii_strcasecmp(dialog_in->theirtag, dialog->theirtag))
+				sipe_strcase_equal(dialog_in->callid, dialog->callid) &&
+				sipe_strcase_equal(dialog_in->ourtag, dialog->ourtag) &&
+				sipe_strcase_equal(dialog_in->theirtag, dialog->theirtag))
 			{
-				purple_debug_info("sipe", "sipe_dialog_find_3 who='%s'\n",
-							  dialog->with ? dialog->with : "");
+				SIPE_DEBUG_INFO("sipe_dialog_find_3 who='%s'",
+						dialog->with ? dialog->with : "");
 				return dialog;
 			}
 		} SIPE_DIALOG_FOREACH_END;
@@ -110,8 +112,8 @@ struct sip_dialog *sipe_dialog_find(struct sip_session *session,
 {
 	if (session && who) {
 		SIPE_DIALOG_FOREACH {
-			if (dialog->with && !g_ascii_strcasecmp(who, dialog->with)) {
-				purple_debug_info("sipe", "sipe_dialog_find who='%s'\n", who);
+			if (dialog->with && sipe_strcase_equal(who, dialog->with)) {
+				SIPE_DEBUG_INFO("sipe_dialog_find who='%s'", who);
 				return dialog;
 			}
 		} SIPE_DIALOG_FOREACH_END;
@@ -123,7 +125,7 @@ void sipe_dialog_remove(struct sip_session *session, const gchar *who)
 {
 	struct sip_dialog *dialog = sipe_dialog_find(session, who);
 	if (dialog) {
-		purple_debug_info("sipe", "sipe_dialog_remove who='%s' with='%s'\n", who, dialog->with ? dialog->with : "");
+		SIPE_DEBUG_INFO("sipe_dialog_remove who='%s' with='%s'", who, dialog->with ? dialog->with : "");
 		session->dialogs = g_slist_remove(session->dialogs, dialog);
 		sipe_dialog_free(dialog);
 	}
@@ -135,8 +137,8 @@ sipe_dialog_remove_3(struct sip_session *session,
 {
 	struct sip_dialog *dialog = sipe_dialog_find_3(session, dialog_in);
 	if (dialog) {
-		purple_debug_info("sipe", "sipe_dialog_remove_3 with='%s'\n",
-					  dialog->with ? dialog->with : "");
+		SIPE_DEBUG_INFO("sipe_dialog_remove_3 with='%s'",
+				dialog->with ? dialog->with : "");
 		session->dialogs = g_slist_remove(session->dialogs, dialog);
 		sipe_dialog_free(dialog);
 	}
@@ -170,13 +172,13 @@ void sipe_dialog_parse_routes(struct sip_dialog *dialog,
 
         while (hdr) {
                 struct sipnameval *elem = hdr->data;
-                if(!g_ascii_strcasecmp(elem->name, "Record-Route")) {
+                if (sipe_strcase_equal(elem->name, "Record-Route")) {
 			gchar **parts = g_strsplit(elem->value, ",", 0);
 			gchar **part = parts;
 
 			while (*part) {
 				gchar *route = sipmsg_find_part_of_header(*part, "<", ">", NULL);
-				purple_debug_info("sipe", "sipe_dialog_parse_routes: route %s \n", route);
+				SIPE_DEBUG_INFO("sipe_dialog_parse_routes: route %s", route);
 				dialog->routes = g_slist_append(dialog->routes, route);
 				part++;
 			}
@@ -214,7 +216,7 @@ sipe_get_supported_header(const struct sipmsg *msg,
 	while(hdr)
 	{
 		elem = hdr->data;
-		if(!g_ascii_strcasecmp(elem->name, "Supported")
+		if (sipe_strcase_equal(elem->name, "Supported")
 			&& !g_slist_find_custom(dialog->supported, elem->value, (GCompareFunc)g_ascii_strcasecmp))
 		{
 			dialog->supported = g_slist_append(dialog->supported, g_strdup(elem->value));
