@@ -28,18 +28,19 @@
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <time.h>
+
+#include <glib.h>
 #include <glib/gprintf.h>
 
-#include "debug.h"
-
-#include "sipe.h"
-#include "sipe-ft.h"
-#include "sipe-dialog.h"
-#include "sipe-nls.h"
-#include "sipe-session.h"
-#include "sipe-utils.h"
+#include "connection.h"
+#include "eventloop.h"
+#include "ft.h"
+#include "network.h"
 
 #ifdef _WIN32
+/* for network */
+#include "libc_interface.h"
 #include <nspapi.h>
 #else
 #include <sys/types.h>
@@ -48,7 +49,21 @@
 #include <netinet/in.h>
 #include <net/if.h>
 #include <arpa/inet.h>
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h> /* SIOCGIFCONF for Solaris */
 #endif
+#endif
+
+#include "sipe-common.h"
+#include "sipmsg.h"
+#include "sip-sec.h"
+#include "sipe-backend.h"
+#include "sipe-dialog.h"
+#include "sipe-nls.h"
+#include "sipe-ft.h"
+#include "sipe-session.h"
+#include "sipe-utils.h"
+#include "sipe.h"
 
 #define SIPE_FT_KEY_LENGTH          24
 #define SIPE_FT_CHUNK_HEADER_LENGTH  3
@@ -73,8 +88,8 @@ struct _sipe_file_transfer {
 	unsigned auth_cookie;
 	struct sipe_account_data *sip;
 	struct sip_dialog *dialog;
-	PurpleCipherContext *cipher_context;
-	PurpleCipherContext *hmac_context;
+	gpointer cipher_context;
+	gpointer hmac_context;
 
 	PurpleNetworkListenData *listener;
 	int listenfd;
@@ -91,9 +106,9 @@ static void send_filetransfer_cancel(PurpleXfer* xfer);
 static ssize_t do_read(PurpleXfer *xfer, guchar *buf, size_t len);
 static gboolean read_fully(PurpleXfer *xfer, guchar *buf, size_t len);
 static gssize read_line(PurpleXfer *xfer, gchar *buffer, gssize size);
-static void sipe_cipher_context_init(PurpleCipherContext **rc4_context, const guchar *enc_key);
-static void sipe_hmac_context_init(PurpleCipherContext **hmac_context, const guchar *hash_key);
-static gchar *sipe_hmac_finalize(PurpleCipherContext *hmac_context);
+static gpointer sipe_cipher_context_init(const guchar *enc_key);
+static gpointer sipe_hmac_context_init(const guchar *hash_key);
+static gchar *sipe_hmac_finalize(gpointer hmac_context);
 static void generate_key(guchar *buffer, gsize size);
 static void set_socket_nonblock(int fd, gboolean state);
 static void sipe_ft_listen_socket_created(int listenfd, gpointer data);
@@ -123,16 +138,16 @@ sipe_ft_free_xfer_struct(PurpleXfer *xfer)
 			xfer->watcher = 0;
 		}
 		if (ft->listenfd >= 0) {
-			purple_debug_info("sipe", "sipe_ft_free_xfer_struct: closing listening socket %d\n", ft->listenfd);
+			SIPE_DEBUG_INFO("sipe_ft_free_xfer_struct: closing listening socket %d", ft->listenfd);
 			close(ft->listenfd);
 		}
 		if (ft->listener)
 			purple_network_listen_cancel(ft->listener);
 		if (ft->cipher_context)
-			purple_cipher_context_destroy(ft->cipher_context);
+			sipe_backend_crypt_ft_destroy(ft->cipher_context);
 
 		if (ft->hmac_context)
-			purple_cipher_context_destroy(ft->hmac_context);
+			sipe_backend_digest_ft_destroy(ft->hmac_context);
 
 		g_free(ft->encrypted_outbuf);
 		g_free(ft->invitation_cookie);
@@ -237,8 +252,8 @@ sipe_ft_incoming_start(PurpleXfer *xfer)
 
 	ft->bytes_remaining_chunk = 0;
 
-	sipe_cipher_context_init(&ft->cipher_context, ft->encryption_key);
-	sipe_hmac_context_init(&ft->hmac_context, ft->hash_key);
+	ft->cipher_context = sipe_cipher_context_init(ft->encryption_key);
+	ft->hmac_context   = sipe_hmac_context_init(ft->hash_key);
 }
 
 static void
@@ -319,8 +334,8 @@ sipe_ft_read(guchar **buffer, PurpleXfer *xfer)
 	*buffer = g_malloc(bytes_to_read);
 	if (!*buffer) {
 		raise_ft_error(xfer, _("Out of memory"));
-		purple_debug_error("sipe", "sipe_ft_read: can't allocate %" G_GSIZE_FORMAT " bytes for receive buffer\n",
-				   bytes_to_read);
+		SIPE_DEBUG_ERROR("sipe_ft_read: can't allocate %" G_GSIZE_FORMAT " bytes for receive buffer",
+				 bytes_to_read);
 		return -1;
 	}
 
@@ -335,17 +350,17 @@ sipe_ft_read(guchar **buffer, PurpleXfer *xfer)
 
 		if (!decrypted) {
 			raise_ft_error(xfer, _("Out of memory"));
-			purple_debug_error("sipe", "sipe_ft_read: can't allocate %" G_GSIZE_FORMAT " bytes for decryption buffer\n",
-					   (gsize)bytes_read);
+			SIPE_DEBUG_ERROR("sipe_ft_read: can't allocate %" G_GSIZE_FORMAT " bytes for decryption buffer",
+					 (gsize)bytes_read);
 			g_free(*buffer);
 			*buffer = NULL;
 			return -1;
 		}
-		purple_cipher_context_encrypt(ft->cipher_context, *buffer, bytes_read, decrypted, NULL);
+		sipe_backend_crypt_ft_stream(ft->cipher_context, *buffer, bytes_read, decrypted);
 		g_free(*buffer);
 		*buffer = decrypted;
 
-		purple_cipher_context_append(ft->hmac_context, decrypted, bytes_read);
+		sipe_backend_digest_ft_update(ft->hmac_context, decrypted, bytes_read);
 
 		ft->bytes_remaining_chunk -= bytes_read;
 	}
@@ -390,17 +405,17 @@ sipe_ft_write(const guchar *buffer, size_t size, PurpleXfer *xfer)
 			ft->encrypted_outbuf = g_malloc(ft->outbuf_size);
 			if (!ft->encrypted_outbuf) {
 				raise_ft_error(xfer, _("Out of memory"));
-				purple_debug_error("sipe", "sipe_ft_write: can't allocate %" G_GSIZE_FORMAT " bytes for send buffer\n",
-						   ft->outbuf_size);
+				SIPE_DEBUG_ERROR("sipe_ft_write: can't allocate %" G_GSIZE_FORMAT " bytes for send buffer",
+						 ft->outbuf_size);
 				return -1;
 			}
 		}
 
 		ft->bytes_remaining_chunk = size;
 		ft->outbuf_ptr = ft->encrypted_outbuf;
-		purple_cipher_context_encrypt(ft->cipher_context, buffer, size,
-					      ft->encrypted_outbuf, NULL);
-		purple_cipher_context_append(ft->hmac_context, buffer, size);
+		sipe_backend_crypt_ft_stream(ft->cipher_context, buffer, size,
+					     ft->encrypted_outbuf);
+		sipe_backend_digest_ft_update(ft->hmac_context, buffer, size);
 
 		/* chunk header format:
 		 *
@@ -503,8 +518,8 @@ sipe_ft_outgoing_start(PurpleXfer *xfer)
 
 	if (!sipe_strequal(buf,VER)) {
 		raise_ft_error_and_cancel(xfer,_("File transfer initialization failed."));
-		purple_debug_info("sipe","File transfer VER string incorrect, received: %s expected: %s",
-				  buf, VER);
+		SIPE_DEBUG_INFO("File transfer VER string incorrect, received: %s expected: %s",
+				buf, VER);
 		return;
 	}
 
@@ -523,11 +538,11 @@ sipe_ft_outgoing_start(PurpleXfer *xfer)
 	auth_cookie_received = g_ascii_strtoull(parts[2],NULL,10);
 
 	// xfer->who has 'sip:' prefix, skip these four characters
-	users_match = !g_ascii_strcasecmp(parts[1], (xfer->who + 4));
+	users_match = sipe_strcase_equal(parts[1], (xfer->who + 4));
 	g_strfreev(parts);
 
-	purple_debug_info("sipe","File transfer authentication: %s Expected: USR %s %u\n",
-			  buf, xfer->who + 4, ft->auth_cookie);
+	SIPE_DEBUG_INFO("File transfer authentication: %s Expected: USR %s %u",
+			buf, xfer->who + 4, ft->auth_cookie);
 
 	if (!users_match || (ft->auth_cookie != auth_cookie_received)) {
 		raise_ft_error_and_cancel(xfer,
@@ -552,8 +567,8 @@ sipe_ft_outgoing_start(PurpleXfer *xfer)
 
 	ft->bytes_remaining_chunk = 0;
 
-	sipe_cipher_context_init(&ft->cipher_context, ft->encryption_key);
-	sipe_hmac_context_init(&ft->hmac_context, ft->hash_key);
+	ft->cipher_context = sipe_cipher_context_init(ft->encryption_key);
+	ft->hmac_context   = sipe_hmac_context_init(ft->hash_key);
 }
 
 static void
@@ -602,7 +617,7 @@ void sipe_ft_incoming_transfer(PurpleAccount *account, struct sipmsg *msg, const
 	}
 
 	if (!session) {
-		purple_debug_error("sipe", "sipe_ft_incoming_transfer: can't find session for remote party\n");
+		SIPE_DEBUG_ERROR_NOFORMAT("sipe_ft_incoming_transfer: can't find session for remote party");
 		return;
 	}
 
@@ -657,7 +672,7 @@ void sipe_ft_incoming_accept(PurpleAccount *account, const GSList *body)
 			ft->auth_cookie = g_ascii_strtoull(auth_cookie,NULL,10);
 		if (enc_key_b64) {
 			gsize ret_len;
-			guchar *enc_key = purple_base64_decode(enc_key_b64, &ret_len);
+			guchar *enc_key = g_base64_decode(enc_key_b64, &ret_len);
 			if (ret_len == SIPE_FT_KEY_LENGTH) {
 				memcpy(ft->encryption_key,enc_key,SIPE_FT_KEY_LENGTH);
 			} else {
@@ -670,7 +685,7 @@ void sipe_ft_incoming_accept(PurpleAccount *account, const GSList *body)
 		}
 		if (hash_key_b64) {
 			gsize ret_len;
-			guchar *hash_key = purple_base64_decode(hash_key_b64, &ret_len);
+			guchar *hash_key = g_base64_decode(hash_key_b64, &ret_len);
 			if (ret_len == SIPE_FT_KEY_LENGTH) {
 				memcpy(ft->hash_key,hash_key,SIPE_FT_KEY_LENGTH);
 			} else {
@@ -714,8 +729,8 @@ static void send_filetransfer_accept(PurpleXfer* xfer)
 	sipe_file_transfer* ft = xfer->data;
 	struct sip_dialog *dialog = ft->dialog;
 
-	gchar *b64_encryption_key = purple_base64_encode(ft->encryption_key,24);
-	gchar *b64_hash_key = purple_base64_encode(ft->hash_key,24);
+	gchar *b64_encryption_key = g_base64_encode(ft->encryption_key, 24);
+	gchar *b64_hash_key = g_base64_encode(ft->hash_key, 24);
 
 	gchar *body = g_strdup_printf("Invitation-Command: ACCEPT\r\n"
 				      "Request-Data: IP-Address:\r\n"
@@ -730,7 +745,7 @@ static void send_filetransfer_accept(PurpleXfer* xfer)
 				      ft->invitation_cookie,
 				      b64_encryption_key,
 				      b64_hash_key
-                                      /*,purple_network_get_my_ip(-1)*/
+                                      /*,sipe_backend_network_ip_address()*/
 		);
 
 	send_sip_request(ft->sip->gc, "MESSAGE", dialog->with, dialog->with,
@@ -814,7 +829,7 @@ static gssize read_line(PurpleXfer *xfer, gchar *buffer, gssize size)
 	return pos;
 }
 
-static void sipe_cipher_context_init(PurpleCipherContext **rc4_context, const guchar *enc_key)
+static gpointer sipe_cipher_context_init(const guchar *enc_key)
 {
 	/*
 	 *      Decryption of file from SIPE file transfer
@@ -824,23 +839,16 @@ static void sipe_cipher_context_init(PurpleCipherContext **rc4_context, const gu
 	 *  2.) Decrypt-Data = RC4 (Encrypt-Data, substr(SHA1-Key, 0, 15)); Decryption of encrypted data, used 16 bytes SHA1-Key;
 	 */
 
-	PurpleCipherContext *sha1_context;
-	guchar k2[20];
+	guchar k2[SIPE_DIGEST_SHA1_LENGTH];
 
 	/* 1.) SHA1 sum	*/
-	sha1_context = purple_cipher_context_new_by_name("sha1", NULL);
-	purple_cipher_context_append(sha1_context, enc_key, SIPE_FT_KEY_LENGTH);
-	purple_cipher_context_digest(sha1_context, sizeof(k2), k2, NULL);
-	purple_cipher_context_destroy(sha1_context);
+        sipe_backend_digest_sha1(enc_key, SIPE_FT_KEY_LENGTH, k2);
 
 	/* 2.) RC4 decryption */
-	*rc4_context = purple_cipher_context_new_by_name("rc4", NULL);
-	purple_cipher_context_set_option(*rc4_context, "key_len", (gpointer)0x10); // only 16 chars key used
-	purple_cipher_context_set_key(*rc4_context, k2);
-
+	return sipe_backend_crypt_ft_start(k2);
 }
 
-static void sipe_hmac_context_init(PurpleCipherContext **hmac_context, const guchar *hash_key)
+static gpointer sipe_hmac_context_init(const guchar *hash_key)
 {
 	/*
 	 * 	Count MAC digest
@@ -850,29 +858,23 @@ static void sipe_hmac_context_init(PurpleCipherContext **hmac_context, const guc
 	 *  2.) MAC = HMAC_SHA1 (Decrypt-Data, substr(HMAC-Key,0,15)); Digest of decrypted file and SHA1-Key (used again only 16 bytes)
 	 */
 
-	PurpleCipherContext *sha1_context;
-	guchar k2[20];
+	guchar k2[SIPE_DIGEST_SHA1_LENGTH];
 
 	/* 1.) SHA1 sum	*/
-	sha1_context = purple_cipher_context_new_by_name("sha1", NULL);
-	purple_cipher_context_append(sha1_context, hash_key, SIPE_FT_KEY_LENGTH);
-	purple_cipher_context_digest(sha1_context, sizeof(k2), k2, NULL);
-	purple_cipher_context_destroy(sha1_context);
+	sipe_backend_digest_sha1(hash_key, SIPE_FT_KEY_LENGTH, k2);
 
 	/* 2.) HMAC (initialization only) */
-	*hmac_context = purple_cipher_context_new_by_name("hmac", NULL);
-	purple_cipher_context_set_option(*hmac_context, "hash", "sha1");
-	purple_cipher_context_set_key_with_len(*hmac_context, k2, 16);
+	return sipe_backend_digest_ft_start(k2);
 }
 
-static gchar* sipe_hmac_finalize(PurpleCipherContext *hmac_context)
+static gchar* sipe_hmac_finalize(gpointer hmac_context)
 {
-	guchar hmac_digest[20];
+	guchar hmac_digest[SIPE_DIGEST_FILETRANSFER_LENGTH];
 
 	/*  MAC = Digest of decrypted file and SHA1-Key (used again only 16 bytes) */
-	purple_cipher_context_digest(hmac_context, sizeof(hmac_digest), hmac_digest, NULL);
+	sipe_backend_digest_ft_end(hmac_context, hmac_digest);
 
-	return purple_base64_encode(hmac_digest, sizeof (hmac_digest));
+	return g_base64_encode(hmac_digest, sizeof (hmac_digest));
 }
 
 static void generate_key(guchar *buffer, gsize size)
