@@ -20,15 +20,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <stdlib.h>
 #include <string.h>
+#include <time.h>
+
 #include <glib.h>
 
-#include "debug.h"
-
-#include "sipe.h"
+#include "sip-sec.h"
+#include "sipe-backend.h"
 #include "sipe-dialog.h"
 #include "sipe-session.h"
 #include "sipe-utils.h"
+#include "sipe.h"
 
 void
 sipe_free_queued_message(struct queued_message *message)
@@ -58,7 +61,7 @@ sipe_session_find_or_add_chat_by_callid(struct sipe_account_data *sip,
 {
 	struct sip_session *session = sipe_session_find_chat_by_callid(sip, callid);
 	if (!session) {
-		purple_debug_info("sipe", "sipe_session_find_or_add_chat_by_callid: new session for %s\n", callid);
+		SIPE_DEBUG_INFO("sipe_session_find_or_add_chat_by_callid: new session for %s", callid);
 		session = sipe_session_add_chat(sip);
 		session->callid = g_strdup(callid);
 	}
@@ -75,7 +78,7 @@ sipe_session_find_chat_by_callid(struct sipe_account_data *sip,
 
 	SIPE_SESSION_FOREACH {
 		if (session->callid &&
-		    !g_ascii_strcasecmp(callid, session->callid)) {
+		    sipe_strcase_equal(callid, session->callid)) {
 			return session;
 		}
 	} SIPE_SESSION_FOREACH_END;
@@ -125,7 +128,7 @@ sipe_session_find_conference(struct sipe_account_data *sip,
 
 	SIPE_SESSION_FOREACH {
 		if (session->focus_uri &&
-		    !g_ascii_strcasecmp(focus_uri, session->focus_uri)) {
+		    sipe_strcase_equal(focus_uri, session->focus_uri)) {
 			return session;
 		}
 	} SIPE_SESSION_FOREACH_END;
@@ -140,7 +143,7 @@ sipe_session_find_im(struct sipe_account_data *sip, const gchar *who)
 	}
 
 	SIPE_SESSION_FOREACH {
-		if (session->with && !g_ascii_strcasecmp(who, session->with)) {
+		if (session->with && sipe_strcase_equal(who, session->with)) {
 			return session;
 		}
 	} SIPE_SESSION_FOREACH_END;
@@ -153,7 +156,7 @@ sipe_session_find_or_add_im(struct sipe_account_data *sip,
 {
 	struct sip_session *session = sipe_session_find_im(sip, who);
 	if (!session) {
-		purple_debug_info("sipe", "sipe_session_find_or_add_im: new session for %s\n", who);
+		SIPE_DEBUG_INFO("sipe_session_find_or_add_im: new session for %s", who);
 		session = g_new0(struct sip_session, 1);
 		session->is_multiparty = FALSE;
 		session->with = g_strdup(who);
