@@ -20,6 +20,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+ * Interface dependencies:
+ *
+ * <glib.h>
+ */
+
+/* Forward declarations */
+struct sipe_account_data;
+struct _PurpleConversation;
+
 /* Helper macros to iterate over session list in a SIP account */
 #define SIPE_SESSION_FOREACH {                             \
 	GSList *entry = sip->sessions;                    \
@@ -34,7 +44,7 @@ struct sip_session {
 	/** key is user (URI) */
 	GSList *dialogs;
 	/** Link to purple chat or IM */
-	PurpleConversation *conv;
+	struct _PurpleConversation *conv;
 	GSList *outgoing_message_queue;
 	/** Key is <Call-ID><CSeq><METHOD><To> */
 	GHashTable *unconfirmed_messages;
