@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2009 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2009-2010 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,15 +20,20 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#include <time.h>
+
 #include <glib.h>
 
-#include "debug.h"
-#include "xmlnode.h"
-
-#include "sipe.h"
+#include "sip-sec.h"
+#include "sipe-backend.h"
+#include "sipe-core.h"    /* to ensure same API for backends */
 #include "sipe-utils.h"
+#include "uuid.h"
+#include "sipe.h"
 
 /* Generate 32 random bits */
 #define RANDOM32BITS (rand() & 0xFFFF)
@@ -65,7 +70,7 @@ gchar *parse_from(const gchar *hdr)
 	const gchar *tmp, *tmp2 = hdr;
 
 	if (!hdr) return NULL;
-	purple_debug_info("sipe", "parsing address out of %s\n", hdr);
+	SIPE_DEBUG_INFO("parsing address out of %s", hdr);
 	tmp = strchr(hdr, '<');
 
 	/* i hate the different SIP UA behaviours... */
@@ -75,7 +80,7 @@ gchar *parse_from(const gchar *hdr)
 		if (tmp) {
 			from = g_strndup(tmp2, tmp - tmp2);
 		} else {
-			purple_debug_info("sipe", "found < without > in From\n");
+			SIPE_DEBUG_INFO_NOFORMAT("found < without > in From");
 			return NULL;
 		}
 	} else {
@@ -86,7 +91,7 @@ gchar *parse_from(const gchar *hdr)
 			from = g_strdup(tmp2);
 		}
 	}
-	purple_debug_info("sipe", "got %s\n", from);
+	SIPE_DEBUG_INFO("got %s", from);
 	return from;
 }
 
@@ -112,59 +117,14 @@ gchar *sip_uri(const gchar *string)
 	return(strstr(string, "sip:") ? g_strdup(string) : sip_uri_from_name(string));
 }
 
-xmlnode *xmlnode_get_descendant(const xmlnode *parent, ...)
-{
-	va_list args;
-	xmlnode *node = NULL;
-	const gchar *name;
-
-	va_start(args, parent);
-	while ((name = va_arg(args, const char *)) != NULL) {
-		node = xmlnode_get_child(parent, name);
-		if (node == NULL) break;
-		parent = node;
-	}
-	va_end(args);
-
-	return node;
-}
-
-gint xmlnode_get_int_attrib(xmlnode *node,
-			    const char *attr,
-			    gint fallback)
-{
-	const char *value = xmlnode_get_attrib(node, attr);
-	return(value ? atoi(value) : fallback);
-}
-
-
-//* @TODO Do we need compat with glib < 2.8 ? */
-char *sipe_get_host_name(void)
-{
-#if GLIB_CHECK_VERSION(2,8,0)
-	const gchar * hostname = g_get_host_name();
-#else
-	static char hostname[256];
-	int ret = gethostname(hostname, sizeof(hostname));
-	hostname[sizeof(hostname) - 1] = '\0';
-	if (ret == -1 || hostname[0] == '\0') {
-		purple_debug(PURPLE_DEBUG_MISC, "sipe", "Error when getting host name.  Using \"localhost.\"\n");
-		g_strerror(errno);
-		strcpy(hostname, "localhost");
-	}
-#endif
-	/*const gchar * hostname = purple_get_host_name();*/
-	return (char *)hostname;
-}
-
 gchar *
 get_epid(struct sipe_account_data *sip)
 {
 	if (!sip->epid) {
 		gchar *self_sip_uri = sip_uri_self(sip);
 		sip->epid = sipe_get_epid(self_sip_uri,
-					  sipe_get_host_name(),
-					  purple_network_get_my_ip(-1));
+					  g_get_host_name(),
+					  sipe_backend_network_ip_address());
 		g_free(self_sip_uri);
 	}
 	return g_strdup(sip->epid);
@@ -231,7 +191,7 @@ sipe_is_bad_alias(const char *uri,
 
 	/* check if alias is just SIP URI but without 'sip:' prefix */
 	uri_alias = sip_uri_from_name(alias);
-	if (!g_ascii_strcasecmp(uri, uri_alias)) {
+	if (sipe_strcase_equal(uri, uri_alias)) {
 		result = TRUE;
 	}
 	g_free(uri_alias);
@@ -294,17 +254,26 @@ sipe_strequal(const gchar *left, const gchar *right)
 #endif
 }
 
-time_t
-sipe_utils_str_to_time(const char *timestamp)
+gboolean
+sipe_strcase_equal(const gchar *left, const gchar *right)
 {
-	return purple_str_to_time(timestamp, TRUE, NULL, NULL, NULL);
+	return ((left == NULL && right == NULL) ||
+	        (left != NULL && right != NULL && g_ascii_strcasecmp(left, right) == 0));
 }
 
-char *
+time_t
+sipe_utils_str_to_time(const gchar *timestamp)
+{
+	GTimeVal time;
+	g_time_val_from_iso8601(timestamp, &time);
+	return time.tv_sec;
+}
+
+gchar *
 sipe_utils_time_to_str(time_t timestamp)
 {
-#define SIPE_XML_DATE_PATTERN	"%Y-%m-%dT%H:%M:%SZ"
-	return g_strdup(purple_utf8_strftime(SIPE_XML_DATE_PATTERN, gmtime(&timestamp)));
+	GTimeVal time = { timestamp, 0 };
+	return g_time_val_to_iso8601(&time);
 }
 
 size_t
@@ -387,8 +356,8 @@ sipe_utils_nameval_add(GSList* list, const gchar *name, const gchar *value)
 
 	/* SANITY CHECK: the calling code must be fixed if this happens! */
 	if (!value) {
-		purple_debug(PURPLE_DEBUG_ERROR, "sipe", "sipe_utils_nameval_add: NULL value for %s\n",
-			     name);
+		SIPE_DEBUG_ERROR("sipe_utils_nameval_add: NULL value for %s",
+				 name);
 		value = "";
 	}
 
@@ -425,7 +394,7 @@ sipe_utils_nameval_find_instance(const GSList *list, const gchar *name, int whic
 	while(tmp) {
 		elem = tmp->data;
 		// OCS2005 can send the same header in either all caps or mixed case
-		if (g_ascii_strcasecmp(elem->name,name)==0) {
+		if (sipe_strcase_equal(elem->name, name)) {
 			if (i == which) {
 				return elem->value;
 			}
@@ -436,6 +405,22 @@ sipe_utils_nameval_find_instance(const GSList *list, const gchar *name, int whic
 	return NULL;
 }
 
+gchar *sipe_utils_str_replace(const gchar *string,
+			      const gchar *delimiter,
+			      const gchar *replacement)
+{
+	gchar **split;
+	gchar *result;
+
+	if (!string || !delimiter || !replacement) return NULL;
+
+	split = g_strsplit(string, delimiter, 0);
+	result = g_strjoinv(replacement, split);
+	g_strfreev(split);
+
+	return result;
+}
+
 /*
   Local Variables:
   mode: c
