@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2009 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2009-2010 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+ * Interface dependencies:
+ *
+ * <time.h>
+ * <glib.h>
+ */
+
+/* Forward declarations */
+struct sipe_account_data;
+
 /* Our publication type keys. OCS 2007+
  * Format: SIPE_PUB_{Category}[_{SubSategory}]
  */
@@ -196,33 +206,6 @@ gboolean
 sipe_is_bad_alias(const char *uri,
 		  const char *alias);
 
-/**
- * Find a XML node from the parent with the specified path
- *
- * @param parent (in) XML node to start search from
- * @param ...    (in) Names of the descendant nodes
- *
- * @return descendant XML node
- */
-xmlnode * xmlnode_get_descendant(const xmlnode *parent, ...);
-
-/**
- * Convert a XML attribute to an integer
- * @param node     (in) XML node
- * @param attr     (in) name of the attribute
- * @param fallback (in) default value if the attribute doesn't exist
- *
- * @return attribute value converted to integer or the fallback value
- */
-gint xmlnode_get_int_attrib(xmlnode *node,
-			    const char *attr,
-			    gint fallback);
-
-/**
- * For glib < 2.8 compatibility
- */
-char *sipe_get_host_name(void);
-
 /**
  * Checks if provided string is empty - NULL, zero size or just series of white spaces.
  * Doesn't modify input string.
@@ -260,6 +243,20 @@ fix_newlines(const char *st);
  */
 gboolean sipe_strequal(const gchar *left, const gchar *right);
 
+/**
+ * Tests two strings for equality, ignoring the case
+ *
+ * Same as glib @c g_ascii_strcasecmp() but works correctly for @c NULL
+ * pointers too. Plus it doesn't complain loudly about them...
+ *
+ * @param left	A string
+ * @param right A string to compare with left
+ *
+ * @return @c TRUE if the strings are the same, else @c FALSE.
+ *
+ */
+gboolean sipe_strcase_equal(const gchar *left, const gchar *right);
+
 /**
  * Parses a timestamp in ISO8601 format and returns a time_t.
  * Assumes UTC if no timezone specified
@@ -267,7 +264,7 @@ gboolean sipe_strequal(const gchar *left, const gchar *right);
  * @param timestamp The timestamp
  */
 time_t
-sipe_utils_str_to_time(const char *timestamp);
+sipe_utils_str_to_time(const gchar *timestamp);
 
 /**
  * Converts time_t to ISO8601 string.
@@ -277,7 +274,7 @@ sipe_utils_str_to_time(const char *timestamp);
  *
  * Example: 2010-02-03T23:59:59Z
  */
-char *
+gchar *
 sipe_utils_time_to_str(time_t timestamp);
 
 struct sipnameval {
@@ -367,3 +364,16 @@ sipe_utils_nameval_find_instance(const GSList *list, const gchar *name, int whic
 void
 sipe_utils_nameval_free(GSList *list);
 
+/**
+ * Given a string, this replaces one substring with another
+ * and returns a newly allocated string.
+ *
+ * @param string      the string from which to replace stuff.
+ * @param delimiter   the substring you want replaced.
+ * @param replacement the substring you want as replacement.
+ *
+ * @return string with the substitution or NULL. Must be g_free()'d after use.
+ */
+gchar *sipe_utils_str_replace(const gchar *string,
+			      const gchar *delimiter,
+			      const gchar *replacement);
