@@ -3,6 +3,7 @@
  *
  * pidgin-sipe
  *
+ * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2008 Novell, Inc.
  * Copyright (C) 2007 Anibal Avelar <avelar@gmail.com>
  * Copyright (C) 2005 Thomas Butter <butter@uni-mannheim.de>
@@ -22,30 +23,27 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef _PIDGIN_SIPE_H
-#define _PIDGIN_SIPE_H
+/*
+ * Interface dependencies:
+ *
+ * <time.h>
+ * <glib.h>
+ * "sip-sec.h"
+ */
 
-#include <glib.h>
-#include <time.h>
-
-#ifdef _WIN32
-#include "internal.h"
-#endif
-
-#include "cipher.h"
-#include "circbuffer.h"
-#include "dnsquery.h"
-#include "dnssrv.h"
-#include "network.h"
-#include "proxy.h"
-#include "prpl.h"
-#include "sslconn.h"
-
-#include "sipmsg.h"
-#include "sip-sec.h"
-#include "uuid.h"
-
-#define SIPE_UNUSED_PARAMETER __attribute__((unused))
+/* Forward declarations */
+struct sipmsg;
+struct _PurpleAccount;
+struct _PurpleCircBuffer;
+struct _PurpleConnection;
+struct _PurpleDnsQueryData;
+struct _PurpleGroup;
+struct _PurpleNetworkListenData;
+struct _PurplePlugin;
+struct _PurpleSrvQueryData;
+struct _PurpleSslConnection;
+struct sip_sec_context;
+enum sip_sec_auth_type;
 
 #define SIMPLE_BUF_INC 4096
 
@@ -109,8 +107,8 @@ struct sipe_buddy {
 };
 
 struct sip_auth {
-	SipSecAuthType type;
-	SipSecContext gssapi_context;
+	enum sip_sec_auth_type type;
+	struct sip_sec_context *gssapi_context;
 	gchar *gssapi_data;
 	gchar *opaque;
 	gchar *realm;
@@ -161,12 +159,12 @@ struct sipe_container {
 /** MS-PRES container member */
 struct sipe_container_member {
 	/** user, domain, sameEnterprise, federated, publicCloud; everyone */
-	const gchar *type;
-	const gchar *value;
+	gchar *type;
+	gchar *value;
 };
 
 struct sipe_account_data {
-	PurpleConnection *gc;
+	struct _PurpleConnection *gc;
 	gchar *sipdomain;
 	gchar *username;
 	gchar *authdomain;
@@ -176,10 +174,10 @@ struct sipe_account_data {
 	gchar *focus_factory_uri;
 	/** Allowed server events to subscribe. From register OK response. */
 	GSList *allow_events;
-	PurpleDnsQueryData *query_data;
-	PurpleSrvQueryData *srv_query_data;
+	struct _PurpleDnsQueryData *query_data;
+	struct _PurpleSrvQueryData *srv_query_data;
 	const struct sipe_service_data *service_data;
-	PurpleNetworkListenData *listen_data;
+	struct _PurpleNetworkListenData *listen_data;
 	int fd;
 	int cseq;
 	time_t last_keepalive;
@@ -219,8 +217,8 @@ struct sipe_account_data {
 	guint keepalive_timeout;
 	GSList *timeouts;
 	gboolean connecting;
-	PurpleAccount *account;
-	PurpleCircBuffer *txbuf;
+	struct _PurpleAccount *account;
+	struct _PurpleCircBuffer *txbuf;
 	guint tx_handler;
 	gchar *regcallid;
 	GSList *transactions;
@@ -230,7 +228,7 @@ struct sipe_account_data {
 	GHashTable *filetransfers;
 	sipe_transport_type transport;
 	gboolean auto_transport;
-	PurpleSslConnection *gsc;
+	struct _PurpleSslConnection *gsc;
 	struct sockaddr *serveraddr;
 	gchar *realhostname;
 	int realport; /* port and hostname from SRV record */
@@ -246,7 +244,7 @@ struct sipe_account_data {
 	 * - OOF flag
 	 * - User status
 	 */
-	xmlnode *user_info;
+	gchar *user_states;
 };
 
 struct sip_connection {
@@ -310,7 +308,7 @@ sipe_schedule_action(const gchar *name,
 struct sipe_group {
 	gchar *name;
 	int id;
-    PurpleGroup *purple_group;
+	struct _PurpleGroup *purple_group;
 };
 
 struct group_user_context {
@@ -320,10 +318,6 @@ struct group_user_context {
 
 GSList * slist_insert_unique_sorted(GSList *list, gpointer data, GCompareFunc func);
 
-GList *sipe_actions(PurplePlugin *plugin, gpointer context);
-
-gboolean purple_init_plugin(PurplePlugin *plugin);
-
 /**
  * Publishes self status
  * based on own calendar information,
@@ -366,11 +360,11 @@ struct sip_dialog;
 
 /* SIP send module? */
 struct transaction *
-send_sip_request(PurpleConnection *gc, const gchar *method,
+send_sip_request(struct _PurpleConnection *gc, const gchar *method,
 		 const gchar *url, const gchar *to, const gchar *addheaders,
 		 const gchar *body, struct sip_dialog *dialog, TransCallback tc);
 void
-send_sip_response(PurpleConnection *gc, struct sipmsg *msg, int code,
+send_sip_response(struct _PurpleConnection *gc, struct sipmsg *msg, int code,
 		  const char *text, const char *body);
 void
 sipe_invite(struct sipe_account_data *sip, struct sip_session *session,
@@ -815,5 +809,3 @@ sipe_im_process_queue (struct sipe_account_data * sip, struct sip_session * sess
 	"</SOAP-ENV:Envelope>"
 
 #define SIPE_SOAP_SEARCH_ROW "<m:row m:attrib=\"%s\" m:value=\"%s\"/>"
-
-#endif /* _PIDGIN_SIPE_H */
