@@ -1,8 +1,9 @@
 /**
  * @file sipmsg.c
  *
- * gaim
+ * pidgin-sipe
  *
+ * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2008 Novell, Inc.
  * Copyright (C) 2005 Thomas Butter <butter@uni-mannheim.de>
  *
@@ -21,15 +22,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
+#include <time.h>
+
 #include <glib.h>
 
-#include "debug.h"
-#include "mime.h"
-
-#include "sipe.h"
-#include "sipe-utils.h"
 #include "sipmsg.h"
+#include "sipe-backend.h"
+#include "sipe-mime.h"
+#include "sipe-utils.h"
 
 struct sipmsg *sipmsg_parse_msg(const gchar *msg) {
 	const char *tmp = strstr(msg, "\r\n\r\n");
@@ -83,7 +86,7 @@ struct sipmsg *sipmsg_parse_header(const gchar *header) {
 	if (contentlength) {
 		msg->bodylen = strtol(contentlength,NULL,10);
 	} else {
-		purple_debug_fatal("sipe", "sipmsg_parse_header(): Content-Length header not found\n");
+		SIPE_DEBUG_FATAL_NOFORMAT("sipmsg_parse_header(): Content-Length header not found");
 	}
 	if(msg->response) {
 		const gchar *tmp;
@@ -101,20 +104,6 @@ struct sipmsg *sipmsg_parse_header(const gchar *header) {
 	return msg;
 }
 
-void sipmsg_print(const struct sipmsg *msg) {
-	GSList *cur;
-	struct sipnameval *elem;
-	purple_debug(PURPLE_DEBUG_MISC, "sipe", "SIP MSG\n");
-	purple_debug(PURPLE_DEBUG_MISC, "sipe", "response: %d\nmethod: %s\nbodylen: %d\n",msg->response,msg->method,msg->bodylen);
-	if(msg->target) purple_debug(PURPLE_DEBUG_MISC, "sipe", "target: %s\n",msg->target);
-	cur = msg->headers;
-	while(cur) {
-		elem = cur->data;
-		purple_debug(PURPLE_DEBUG_MISC, "sipe", "name: %s value: %s\n",elem->name, elem->value);
-		cur = g_slist_next(cur);
-	}
-}
-
 char *sipmsg_to_string(const struct sipmsg *msg) {
 	GSList *cur;
 	GString *outstr = g_string_new("");
@@ -153,8 +142,8 @@ void sipmsg_add_header_now_pos(struct sipmsg *msg, const gchar *name, const gcha
 
 	/* SANITY CHECK: the calling code must be fixed if this happens! */
 	if (!value) {
-		purple_debug(PURPLE_DEBUG_ERROR, "sipe", "sipmsg_add_header_now_pos: NULL value for %s (%d)\n",
-			     name, pos);
+		SIPE_DEBUG_ERROR("sipmsg_add_header_now_pos: NULL value for %s (%d)",
+				 name, pos);
 		value = "";
 	}
 
@@ -171,8 +160,8 @@ void sipmsg_add_header_now(struct sipmsg *msg, const gchar *name, const gchar *v
 
 	/* SANITY CHECK: the calling code must be fixed if this happens! */
 	if (!value) {
-		purple_debug(PURPLE_DEBUG_ERROR, "sipe", "sipmsg_add_header_now: NULL value for %s\n",
-			     name);
+		SIPE_DEBUG_ERROR("sipmsg_add_header_now: NULL value for %s",
+				 name);
 		value = "";
 	}
 
@@ -189,8 +178,7 @@ void sipmsg_add_header(struct sipmsg *msg, const gchar *name, const gchar *value
 
 	/* SANITY CHECK: the calling code must be fixed if this happens! */
 	if (!value) {
-		purple_debug(PURPLE_DEBUG_ERROR, "sipe", "sipmsg_add_header: NULL value for %s\n",
-			     name);
+		SIPE_DEBUG_ERROR("sipmsg_add_header: NULL value for %s", name);
 		value = "";
 	}
 
@@ -220,9 +208,9 @@ void sipmsg_strip_headers(struct sipmsg *msg, const gchar *keepers[]) {
 			i++;
 		}
 
-		if (!keeper) {
+		if (!keeper) {      
 			GSList *to_delete = entry;
-			purple_debug_info("sipe", "sipmsg_strip_headers: removing %s\n", elem->name);
+			SIPE_DEBUG_INFO("sipmsg_strip_headers: removing %s", elem->name);
 			entry = g_slist_next(entry);
 			msg->headers = g_slist_delete_link(msg->headers, to_delete);
 			g_free(elem->name);
@@ -262,7 +250,7 @@ void sipmsg_remove_header_now(struct sipmsg *msg, const gchar *name) {
 	while(tmp) {
 		elem = tmp->data;
 		// OCS2005 can send the same header in either all caps or mixed case
-		if (g_ascii_strcasecmp(elem->name, name)==0) {
+		if (sipe_strcase_equal(elem->name, name)) {
 			msg->headers = g_slist_remove(msg->headers, elem);
 			g_free(elem->name);
 			g_free(elem->value);
@@ -364,19 +352,19 @@ gchar *sipmsg_find_auth_header(struct sipmsg *msg, const gchar *name) {
 	tmp = msg->headers;
 	while(tmp) {
 		elem = tmp->data;
-		//purple_debug(PURPLE_DEBUG_MISC, "sipmsg", "Current header: %s\r\n", elem->value);
+		/* SIPE_DEBUG_INFO("Current header: %s", elem->value); */
 		if (elem && elem->name &&
-				(!g_ascii_strcasecmp(elem->name,"WWW-Authenticate")
-					|| !g_ascii_strcasecmp(elem->name,"Authentication-Info")) ) {
+		    (sipe_strcase_equal(elem->name,"WWW-Authenticate") ||
+		     sipe_strcase_equal(elem->name,"Authentication-Info")) ) {
 			if (!g_strncasecmp((gchar *)elem->value, name, name_len)) {
-				//purple_debug(PURPLE_DEBUG_MISC, "sipmsg", "elem->value: %s\r\n", elem->value);
+				/* SIPE_DEBUG_INFO("elem->value: %s", elem->value); */
 				return elem->value;
 			}
 		}
-		//purple_debug(PURPLE_DEBUG_MISC, "sipmsg", "moving to next header\r\n");
+		/* SIPE_DEBUG_INFO_NOFORMAT("moving to next header"); */
 		tmp = g_slist_next(tmp);
 	}
-	purple_debug(PURPLE_DEBUG_MISC, "sipmsg", "Did not found auth header %s\r\n", name);
+	SIPE_DEBUG_INFO("auth header '%s' not found.", name);
 	return NULL;
 }
 
@@ -397,7 +385,7 @@ gchar *sipmsg_get_x_mms_im_format(gchar *msgr) {
 		msgr2 = g_strdup_printf("%s=", msgr2);
 		g_free(tmp_msgr2);
 	}
-	msgr_dec64 = purple_base64_decode(msgr2, &msgr_dec64_len);
+	msgr_dec64 = g_base64_decode(msgr2, &msgr_dec64_len);
 	msgr_utf8 = g_convert((gchar *) msgr_dec64, msgr_dec64_len, "UTF-8", "UTF-16LE", NULL, NULL, NULL);
 	g_free(msgr_dec64);
 	g_free(msgr2);
@@ -429,7 +417,7 @@ gchar *sipmsg_get_msgr_string(gchar *x_mms_im_format) {
 	msgr_orig = g_strdup_printf("X-MMS-IM-Format: %s\r\n\r\n", x_mms_im_format);
 	msgr_utf16 = g_convert(msgr_orig, -1, "UTF-16LE", "UTF-8", NULL, &msgr_utf16_len, NULL);
 	g_free(msgr_orig);
-	msgr_enc = purple_base64_encode((guchar *) msgr_utf16, msgr_utf16_len);
+	msgr_enc = g_base64_encode((guchar *) msgr_utf16, msgr_utf16_len);
 	g_free(msgr_utf16);
 	len = strlen(msgr_enc);
 	while (msgr_enc[len - 1] == '=') len--;
@@ -452,6 +440,41 @@ gchar *sipmsg_apply_x_mms_im_format(const char *x_mms_im_format, gchar *body) {
 	return res;
 }
 
+struct html_message_data {
+	gchar *ms_text_format;
+	gchar *body;
+	gboolean preferred;
+};
+
+static void get_html_message_mime_cb(gpointer user_data,
+				     const gchar *type,
+				     const gchar *body,
+				     gsize length)
+{
+	struct html_message_data *data = user_data;
+
+	if (!data->preferred) {
+		gboolean copy = FALSE;
+
+		/* preferred format */
+		if (g_str_has_prefix(type, "text/html")) {
+			copy = TRUE;
+			data->preferred = TRUE;
+
+		/* fallback format */
+		} else if (g_str_has_prefix(type, "text/plain")) {
+			copy = TRUE;
+		}
+
+		if (copy) {
+			g_free(data->ms_text_format);
+			g_free(data->body);
+			data->ms_text_format = g_strdup(type);
+			data->body = g_strndup(body, length);
+		}
+	}
+}
+
 /* ms-text-format: text/plain; charset=UTF-8;msgr=WAAtAE0...DIADQAKAA0ACgA;ms-body=SGk= */
 gchar *get_html_message(const gchar *ms_text_format_in, const gchar *body_in)
 {
@@ -462,36 +485,14 @@ gchar *get_html_message(const gchar *ms_text_format_in, const gchar *body_in)
 
 	if (g_str_has_prefix(ms_text_format_in, "multipart/related") ||
 	    g_str_has_prefix(ms_text_format_in, "multipart/alternative")) {
-		char *doc = g_strdup_printf("Content-Type: %s\r\n\r\n%s", ms_text_format_in, body_in);
-		PurpleMimeDocument *mime;
-		GList* parts;
+		struct html_message_data data = { NULL, NULL, FALSE };
 
-		mime = purple_mime_document_parse(doc);
-		parts = purple_mime_document_get_parts(mime);
-		while (parts) {
-			const gchar *content_type = purple_mime_part_get_field(parts->data, "Content-Type");
-			if (content_type) {
-				const gchar *content = purple_mime_part_get_data(parts->data);
-				guint length = purple_mime_part_get_length(parts->data);
+		sipe_mime_parts_foreach(ms_text_format_in, body_in,
+					get_html_message_mime_cb, &data);
+
+		ms_text_format = data.ms_text_format;
+		body = data.body;
 
-				/* if no other format has stored */
-				if (!ms_text_format && g_str_has_prefix(content_type, "text/plain")) {
-					ms_text_format = g_strdup(content_type);
-					body = g_strndup(content, length);
-				/* preferred format */
-				} else if (g_str_has_prefix(ms_text_format, "text/html")) {
-					g_free(ms_text_format);
-					g_free(body);
-					ms_text_format = g_strdup(content_type);
-					body = g_strndup(content, length);
-					break;
-				}
-			}
-			parts = parts->next;
-		}
-		g_free(doc);
-		if (mime)
-			purple_mime_document_free(mime);
 	} else {
 		ms_text_format = g_strdup(ms_text_format_in);
 		body = g_strdup(body_in);
@@ -501,11 +502,12 @@ gchar *get_html_message(const gchar *ms_text_format_in, const gchar *body_in)
 		res = body;
 	} else {
 		gchar *tmp = sipmsg_find_part_of_header(ms_text_format, "ms-body=", NULL, NULL);
+		gsize len;
 		if (!tmp) {
 			g_free(ms_text_format);
 			return NULL;
 		}
-		res = (gchar *) purple_base64_decode(tmp, NULL);
+		res = (gchar *) g_base64_decode(tmp, &len);
 		g_free(tmp);
 		if (!res) {
 			g_free(ms_text_format);
@@ -542,11 +544,48 @@ gchar *get_html_message(const gchar *ms_text_format_in, const gchar *body_in)
 //TEMP solution to include it here (copy from purple's msn protocol
 //How to reuse msn's util methods from sipe?
 
-// from internal.h for linux compilation
-#ifndef _WIN32
+/* from internal.h */
 #define MSG_LEN 2048
 #define BUF_LEN MSG_LEN
+
+static
+gchar *sipmsg_uri_unescape(const gchar *string)
+{
+	gchar *unescaped, *tmp;
+
+	if (!string) return(NULL);
+
+#if GLIB_CHECK_VERSION(2,16,0)
+	unescaped = g_uri_unescape_string(string, NULL);
+#else
+	/* loosely based on libpurple/util.c:purple_url_decode() */
+	{
+		gsize i = 0;
+		gsize len = strlen(string);
+
+		unescaped = g_malloc(len + 1);
+		while (len-- > 0) {
+			gchar c = *string++;
+			if ((len >= 2) && (c == '%')) {
+				char hex[3];
+				strncpy(hex, string, 2);
+				hex[2] = '\0';
+				c = strtol(hex, NULL, 16);
+				string += 2;
+				len -= 2;
+			}
+			unescaped[i++] = c;
+		}
+		unescaped[i] = '\0';
+	}
 #endif
+
+	if (!g_utf8_validate(unescaped, -1, (const gchar **)&tmp))
+		*tmp = '\0';
+
+	return(unescaped);
+}
+
 void
 msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
 {
@@ -624,9 +663,12 @@ msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
 				colors[0] = temp;
 			}
 
+			/* hh is undefined in mingw's gcc 4.4
+			 *  https://sourceforge.net/tracker/index.php?func=detail&aid=2818436&group_id=2435&atid=102435
+			 */
 			g_snprintf(tag, sizeof(tag),
-					   "<FONT COLOR=\"#%02hhx%02hhx%02hhx\">",
-					   colors[0], colors[1], colors[2]);
+					   "<FONT COLOR=\"#%02x%02x%02x\">",
+					   (unsigned char)colors[0], (unsigned char)colors[1], (unsigned char)colors[2]);
 
 			pre = g_string_append(pre, tag);
 			post = g_string_prepend(post, "</FONT>");
@@ -645,7 +687,7 @@ msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
 		}
 	}
 
-	cur = g_strdup(purple_url_decode(pre->str));
+	cur = sipmsg_uri_unescape(pre->str);
 	g_string_free(pre, TRUE);
 
 	if (pre_ret != NULL)
@@ -653,7 +695,7 @@ msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
 	else
 		g_free(cur);
 
-	cur = g_strdup(purple_url_decode(post->str));
+	cur = sipmsg_uri_unescape(post->str);
 	g_string_free(post, TRUE);
 
 	if (post_ret != NULL)
@@ -806,7 +848,7 @@ msn_import_html(const char *html, char **attributes, char **message)
 							char *css_attributes;
 							char *attr_dir;
 							css_attributes = g_strndup(c, attr_len);
-							attr_dir = purple_markup_get_css_property(css_attributes, "direction");
+							attr_dir = sipe_backend_markup_css_property(css_attributes, "direction");
 							g_free(css_attributes);
 							if (attr_dir && (!g_ascii_strncasecmp(attr_dir, "RTL", 3)))
 								direction = '1';
