@@ -1,8 +1,9 @@
 /**
  * @file sipmsg.h
  *
- * gaim
+ * pidgin-sipe
  *
+ * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2008 Novell, Inc.
  * Copyright (C) 2005, Thomas Butter <butter@uni-mannheim.de>
  *
@@ -21,10 +22,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef _PURPLE_SIPMSG_H
-#define _PURPLE_SIPMSG_H
-
-#include <glib.h>
+/*
+ * Interface dependencies:
+ *
+ * <glib.h>
+ */
 
 struct sipmsg {
 	int response; /* 0 means request, otherwise response code */
@@ -47,7 +49,6 @@ struct sipendpoint {
 
 struct sipmsg *sipmsg_parse_msg(const gchar *msg);
 struct sipmsg *sipmsg_parse_header(const gchar *header);
-void sipmsg_print(const struct sipmsg *msg);
 void sipmsg_add_header_now(struct sipmsg *msg, const gchar *name, const gchar *value);
 void sipmsg_add_header(struct sipmsg *msg, const gchar *name, const gchar *value);
 void sipmsg_add_header_now_pos(struct sipmsg *msg, const gchar *name, const gchar *value, int pos);
@@ -107,4 +108,3 @@ void sipe_parse_html(const char *html, char **attributes, char **message);
 
 void msn_parse_format(const char *mime, char **pre_ret, char **post_ret);
 void msn_import_html(const char *html, char **attributes, char **message);
-#endif /* _PURPLE_SIPMSG_H */
